From: tip-bot for Toshi Kani <tipbot@zytor.com>
To: linux-tip-commits@vger.kernel.org
Cc: linux-kernel@vger.kernel.org, elliott@hpe.com, hpa@zytor.com,
toshi.kani@hpe.com, bp@alien8.de, konrad.wilk@oracle.com,
mingo@redhat.com, jgross@suse.com, mingo@kernel.org,
akpm@linux-foundation.org, tglx@linutronix.de
Subject: [tip:x86/mm] x86/mm: Fix try_preserve_large_page() to handle large PAT bit
Date: Tue, 22 Sep 2015 12:45:17 -0700 [thread overview]
Message-ID: <tip-3a19109efbfa7d887996a74257556a46e00525c2@git.kernel.org> (raw)
In-Reply-To: <1442514264-12475-10-git-send-email-toshi.kani@hpe.com>
Commit-ID: 3a19109efbfa7d887996a74257556a46e00525c2
Gitweb: http://git.kernel.org/tip/3a19109efbfa7d887996a74257556a46e00525c2
Author: Toshi Kani <toshi.kani@hpe.com>
AuthorDate: Thu, 17 Sep 2015 12:24:22 -0600
Committer: Thomas Gleixner <tglx@linutronix.de>
CommitDate: Tue, 22 Sep 2015 21:27:33 +0200
x86/mm: Fix try_preserve_large_page() to handle large PAT bit
try_preserve_large_page() is called from __change_page_attr() to
change the mapping attribute of a given large page. This function
uses pte_pfn() and pte_pgprot() for PUD/PMD, which do not handle
the large PAT bit properly.
Fix try_preserve_large_page() by using the corresponding pud/pmd
prot/pfn interfaces.
Also remove '#ifdef CONFIG_X86_64', which is not necessary.
Signed-off-by: Toshi Kani <toshi.kani@hpe.com>
Cc: Andrew Morton <akpm@linux-foundation.org>
Cc: Juergen Gross <jgross@suse.com>
Cc: H. Peter Anvin <hpa@zytor.com>
Cc: Ingo Molnar <mingo@redhat.com>
Cc: Borislav Petkov <bp@alien8.de>
Cc: Konrad Wilk <konrad.wilk@oracle.com>
Cc: Robert Elliot <elliott@hpe.com>
Cc: linux-mm@kvack.org
Link: http://lkml.kernel.org/r/1442514264-12475-10-git-send-email-toshi.kani@hpe.com
Signed-off-by: Thomas Gleixner <tglx@linutronix.de>
---
arch/x86/mm/pageattr.c | 24 ++++++++++++++----------
1 file changed, 14 insertions(+), 10 deletions(-)
diff --git a/arch/x86/mm/pageattr.c b/arch/x86/mm/pageattr.c
index 1441368..3f61271 100644
--- a/arch/x86/mm/pageattr.c
+++ b/arch/x86/mm/pageattr.c
@@ -468,7 +468,7 @@ static int
try_preserve_large_page(pte_t *kpte, unsigned long address,
struct cpa_data *cpa)
{
- unsigned long nextpage_addr, numpages, pmask, psize, addr, pfn;
+ unsigned long nextpage_addr, numpages, pmask, psize, addr, pfn, old_pfn;
pte_t new_pte, old_pte, *tmp;
pgprot_t old_prot, new_prot, req_prot;
int i, do_split = 1;
@@ -488,17 +488,21 @@ try_preserve_large_page(pte_t *kpte, unsigned long address,
switch (level) {
case PG_LEVEL_2M:
-#ifdef CONFIG_X86_64
+ old_prot = pmd_pgprot(*(pmd_t *)kpte);
+ old_pfn = pmd_pfn(*(pmd_t *)kpte);
+ break;
case PG_LEVEL_1G:
-#endif
- psize = page_level_size(level);
- pmask = page_level_mask(level);
+ old_prot = pud_pgprot(*(pud_t *)kpte);
+ old_pfn = pud_pfn(*(pud_t *)kpte);
break;
default:
do_split = -EINVAL;
goto out_unlock;
}
+ psize = page_level_size(level);
+ pmask = page_level_mask(level);
+
/*
* Calculate the number of pages, which fit into this large
* page starting at address:
@@ -514,7 +518,7 @@ try_preserve_large_page(pte_t *kpte, unsigned long address,
* up accordingly.
*/
old_pte = *kpte;
- old_prot = req_prot = pgprot_large_2_4k(pte_pgprot(old_pte));
+ old_prot = req_prot = pgprot_large_2_4k(old_prot);
pgprot_val(req_prot) &= ~pgprot_val(cpa->mask_clr);
pgprot_val(req_prot) |= pgprot_val(cpa->mask_set);
@@ -540,10 +544,10 @@ try_preserve_large_page(pte_t *kpte, unsigned long address,
req_prot = canon_pgprot(req_prot);
/*
* old_pte points to the large page base address. So we need
* to add the offset of the virtual address:
*/
- pfn = pte_pfn(old_pte) + ((address & (psize - 1)) >> PAGE_SHIFT);
+ pfn = old_pfn + ((address & (psize - 1)) >> PAGE_SHIFT);
cpa->pfn = pfn;
new_prot = static_protections(req_prot, address, pfn);
@@ -554,7 +558,7 @@ try_preserve_large_page(pte_t *kpte, unsigned long address,
* the pages in the range we try to preserve:
*/
addr = address & pmask;
- pfn = pte_pfn(old_pte);
+ pfn = old_pfn;
for (i = 0; i < (psize >> PAGE_SHIFT); i++, addr += PAGE_SIZE, pfn++) {
pgprot_t chk_prot = static_protections(req_prot, addr, pfn);
@@ -584,7 +588,7 @@ try_preserve_large_page(pte_t *kpte, unsigned long address,
* The address is aligned and the number of pages
* covers the full page.
*/
- new_pte = pfn_pte(pte_pfn(old_pte), new_prot);
+ new_pte = pfn_pte(old_pfn, new_prot);
__set_pmd_pte(kpte, address, new_pte);
cpa->flags |= CPA_FLUSHTLB;
do_split = 0;
next prev parent reply other threads:[~2015-09-22 19:46 UTC|newest]
Thread overview: 52+ messages / expand[flat|nested] mbox.gz Atom feed top
2015-09-17 18:24 [PATCH v4 RESEND 0/11] x86/mm: Handle large PAT bit in pud/pmd interfaces Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 1/11] x86/vdso32: Define PGTABLE_LEVELS to 32bit VDSO Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:42 ` [tip:x86/mm] " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 2/11] x86/asm: Move PUD_PAGE macros to page_types.h Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:42 ` [tip:x86/mm] " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 3/11] x86/asm: Add pud/pmd mask interfaces to handle large PAT bit Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:43 ` [tip:x86/mm] x86/asm: Add pud/ pmd " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 4/11] x86/asm: Fix pud/pmd " Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:43 ` [tip:x86/mm] x86/asm: Fix pud/ pmd " tip-bot for Toshi Kani
2015-11-09 18:06 ` [PATCH v4 RESEND 4/11] x86/asm: Fix pud/pmd " Boris Ostrovsky
2015-11-09 19:16 ` Toshi Kani
2015-11-09 19:16 ` Toshi Kani
2015-11-09 19:39 ` Boris Ostrovsky
2015-11-09 19:39 ` Boris Ostrovsky
2015-11-09 20:47 ` Kirill A. Shutemov
2015-11-09 20:47 ` Kirill A. Shutemov
2015-11-09 22:36 ` Boris Ostrovsky
2015-11-09 22:36 ` Boris Ostrovsky
2015-11-09 23:18 ` Toshi Kani
2015-11-09 23:18 ` Toshi Kani
2015-11-10 0:40 ` Toshi Kani
2015-11-10 0:40 ` Toshi Kani
2015-11-09 20:10 ` Kirill A. Shutemov
2015-11-09 20:10 ` Kirill A. Shutemov
2015-11-09 21:01 ` Toshi Kani
2015-11-09 21:01 ` Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 5/11] x86/asm: Add pud_pgprot() and pmd_pgprot() Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:43 ` [tip:x86/mm] " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 6/11] x86/mm: Fix page table dump to show PAT bit Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:44 ` [tip:x86/mm] " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 7/11] x86/mm: Fix slow_virt_to_phys() to handle large " Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:44 ` [tip:x86/mm] " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 8/11] x86/mm: Fix gup_huge_p?d() " Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:44 ` [tip:x86/mm] " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 9/11] x86/mm: Fix try_preserve_large_page() " Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:45 ` tip-bot for Toshi Kani [this message]
2015-09-17 18:24 ` [PATCH v4 RESEND 10/11] x86/mm: Fix __split_large_page() " Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:45 ` [tip:x86/mm] " tip-bot for Toshi Kani
2015-09-17 18:24 ` [PATCH v4 RESEND 11/11] x86/mm: Fix no-change case in try_preserve_large_page() Toshi Kani
2015-09-17 18:24 ` Toshi Kani
2015-09-22 19:45 ` [tip:x86/mm] " tip-bot for Toshi Kani
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=tip-3a19109efbfa7d887996a74257556a46e00525c2@git.kernel.org \
--to=tipbot@zytor.com \
--cc=akpm@linux-foundation.org \
--cc=bp@alien8.de \
--cc=elliott@hpe.com \
--cc=hpa@zytor.com \
--cc=jgross@suse.com \
--cc=konrad.wilk@oracle.com \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-tip-commits@vger.kernel.org \
--cc=mingo@kernel.org \
--cc=mingo@redhat.com \
--cc=tglx@linutronix.de \
--cc=toshi.kani@hpe.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -540,10 +544,10 @@ try_preserve_large_page(pte_t *kpte, unsigned long address,
req_prot = canon_pgprot(req_prot);
/*
- * old_pte points to the large page base address. So we need
+ * old_pfn points to the large page base pfn. So we need
* to add the offset of the virtual address:
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.